/*
 * Copyright (c) 2014 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*global define, window, document, history, tizen, console*/

/**
 * Init page module.
 *
 * @module views/init
 * @requires {@link core/event}
 * @requires {@link core/application}
 * @requires {@link core/systeminfo}
 * @requires {@link views/main}
 * @requires {@link views/options}
 * @requires {@link views/popup}
 * @namespace views/init
 * @memberof views
 */

define({
    name: 'views/init',
    requires: [
        'core/event',
        'core/application',
        'core/systeminfo',
        'views/main',
        'views/options',
        'views/popup'
    ],
    def: function viewsInit(req) {
        'use strict';

        /**
         * Event module.
         *
         * @private
         * @type {Module}
         */
        var e = req.core.event,

            /**
             * Application module.
             *
             * @private
             * @type {Module}
             */
            app = req.core.application,

            /**
             * Systeminfo module.
             *
             * @private
             * @type {Module}
             */
            sysInfo = req.core.systeminfo,

            /**
             * Popup module.
             *
             * @private
             * @type {Module}
             */
            popup = req.views.popup,

            /**
             * Boolean flag indicating that the application
             * is in progress of closing.
             *
             * @private
             * @type {boolean}
             */
            alreadyClosing = false;

        /**
         * Handles tizenhwkey event.
         *
         * @private
         * @param {Event} ev
         */
        function onHardwareKeysTap(ev) {
            var keyName = ev.keyName,
                page = document.getElementsByClassName('ui-page-active')[0],
                pageId = page ? page.id : '';

            if (keyName === 'back') {
                if (pageId === 'main') {
                    app.exit();
                } else {
                    history.back();
                }
            }
        }

        /**
         * Handles visibilitychange event.
         *
         * @private
         * @fires views.init.visibility.change
         */
        function onVisibilityChange() {
            e.fire('visibility.change');
        }

        /**
         * Handles core.systeminfo.battery.low event.
         *
         * @private
         */
        function onLowBattery() {
            app.exit();
        }

        /**
         * Begins closing operation when USB charger is connected to the device.
         *
         * @private
         */
        function exitIfCharging() {
            if (!alreadyClosing) {
                try {
                    sysInfo.getSystemProperty(
                        'BATTERY',
                        function onValue(battery) {
                            if (!battery.isCharging) {
                                return;
                            }
                            alreadyClosing = true;
                            popup.openExitPopup(
                                'This application only runs without charger.' +
                                ' Please unplug your USB charger.'
                            );
                        },
                        function onError() {
                            return;
                        }
                    );
                } catch (error) {
                    console.warn(
                        'Information about battery state cannot be obtained.',
                        error
                    );
                }
            }
        }

        /**
         * Adds listener to check whether USB charger
         * is connected to the device.
         *
         * @private
         */
        function addUSBListener() {
            try {
                tizen.systeminfo.addPropertyValueChangeListener(
                    'BATTERY',
                    function onValue(battery) {
                        if (battery.isCharging) {
                            exitIfCharging();
                        }
                    }
                );
            } catch (error) {
                console.warn('Property change listener was not set.', error);
            }
        }

        /**
         * Handles views.popup.battery.charger.off event.
         *
         * @private
         */
        function onBatteryChargerOff() {
            alreadyClosing = false;
        }

        /**
         * Registers event listeners.
         *
         * @private
         */
        function bindEvents() {
            window.addEventListener('tizenhwkey', onHardwareKeysTap);
            document.addEventListener('visibilitychange', onVisibilityChange);
            sysInfo.listenBatteryLowState();
            addUSBListener();
        }

        /**
         * Initializes module.
         *
         * @memberof views/init
         * @public
         */
        function init() {
            bindEvents();
            sysInfo.checkBatteryLowState();
            exitIfCharging();
        }

        e.listeners({
            'core.systeminfo.battery.low': onLowBattery,
            'views.popup.battery.charger.off': onBatteryChargerOff
        });

        return {
            init: init
        };
    }

});
