/*
 * Copyright (c) 2014 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*global define, document, window*/

/**
 * Main page module.
 *
 * @module views/main
 * @requires {@link core/event}
 * @requires {@link models/options}
 * @namespace views/main
 * @memberof views
 */

define({
    name: 'views/main',
    requires: [
        'core/event',
        'models/options'
    ],
    def: function viewsMain(req) {
        'use strict';

        /**
         * Event module.
         *
         * @private
         * @type {Module}
         */
        var e = req.core.event,

            /**
             * Options module.
             *
             * @private
             * @type {Module}
             */
            options = req.models.options,

            /**
             * Additional line width.
             *
             * @private
             * @const {number}
             */
            ADDITIONAL_LINE_WIDTH = 0.5,

            /**
             * Half pixel.
             *
             * @private
             * @const {number}
             */
            HALF_PIXEL = 0.5,

            /**
             * Page element.
             *
             * @private
             * @type {HTMLElement}
             */
            page = null,

            /**
             * Content element.
             *
             * @private
             * @type {HTMLElement}
             */
            content = null,

            /**
             * Header element.
             *
             * @private
             * @type {HTMLElement}
             */
            header = null,

            /**
             * Footer element.
             *
             * @private
             * @type {HTMLElement}
             */
            footer = null,

            /**
             * Options button element.
             *
             * @private
             * @type {HTMLElement}
             */
            optionsBtn = null,

            /**
             * Clear button element.
             *
             * @private
             * @type {HTMLElement}
             */
            clearBtn = null,

            /**
             * Canvas element.
             *
             * @private
             * @type {HTMLElement}
             */
            canvas = null,

            /**
             * Canvas context.
             *
             * @private
             * @type {CanvasRenderingContext2D}
             */
            context = null,

            /**
             * Object storing information about all touches,
             * which occur on the device screen.
             *
             * @private
             * @type {object}
             */
            drawPath = {},

            /**
             * Stroke width value.
             *
             * @private
             * @type {number}
             */
            strokeWidth = null,

            /**
             * Stroke color value.
             *
             * @private
             * @type {string}
             */
            strokeColor = null;

        /**
         * Updates stroke properties.
         *
         * @private
         */
        function updateStrokeProperties() {
            strokeWidth = options.getStrokeWidth();
            strokeColor = options.getStrokeColor();
        }

        /**
         * Handles pageshow event.
         *
         * Updates stroke properties.
         *
         * @private
         */
        function onPageShow() {
            updateStrokeProperties();
        }

        /**
         * Handles click event on option button.
         *
         * @private
         * @fires views.main.show.options
         */
        function onOptionsBtnTap() {
            e.fire('show.options');
        }

        /**
         * Handles click event on clear button.
         *
         * Clears the canvas object.
         *
         * @private
         */
        function onClearBtnTap() {
            context.clearRect(0, 0, canvas.width, canvas.height);
        }

        /**
         * Handles touchstart event on canvas.
         *
         * Adds new touch object to the drawPath associative array
         * indexed by touch id. Information about touches are used later
         * to draw lines on touchmove event.
         *
         * Draws dot in place where the screen was touched.
         *
         * The size and color of the dot corresponds to values
         * of the strokeColor and strokeWidth variables.
         *
         * @private
         * @param {Event} ev
         */
        function onCanvasTouchStart(ev) {
            var touch = ev.changedTouches[0];

            drawPath[touch.identifier] = touch;

            context.fillStyle = strokeColor;
            context.beginPath();
            context.arc(
                drawPath[touch.identifier].pageX - canvas.offsetLeft,
                drawPath[touch.identifier].pageY - canvas.offsetTop -
                    header.offsetHeight,
                strokeWidth / 2,
                0,
                Math.PI * 2,
                true
            );
            context.closePath();
            context.fill();
        }

        /**
         * Handles touchend event on canvas.
         *
         * Removes current touch from the drawPath associative array.
         * The information about this touch are no longer used
         * to draw the line on touchmove event.
         *
         * @private
         * @param {Event} ev
         */
        function onCanvasTouchEnd(ev) {
            var touch = ev.changedTouches[0];

            delete drawPath[touch.identifier];
        }

        /**
         * Handles touchmove event on canvas.
         *
         * Iterates through the drawPath associative array
         * indexed by touch id. Draws line for each touch,
         * from previous to the current position of each touch on the screen.
         *
         * The size and color of the line corresponds to values
         * of the strokeColor and strokeWidth variables.
         *
         * @private
         * @param {Event} ev
         */
        function onCanvasTouchMove(ev) {
            var touches = ev.changedTouches,
                touchesLength = touches.length,
                currentDrawPath = null,
                i = 0;

            context.lineWidth = strokeWidth + ADDITIONAL_LINE_WIDTH;
            context.strokeStyle = strokeColor;
            context.lineJoin = 'round';

            for (i = 0; i < touchesLength; i += 1) {
                currentDrawPath = drawPath[touches[i].identifier];
                if (currentDrawPath !== undefined) {
                    context.beginPath();
                    context.moveTo(
                        currentDrawPath.pageX - canvas.offsetLeft + HALF_PIXEL,
                        currentDrawPath.pageY - canvas.offsetTop + HALF_PIXEL -
                            header.offsetHeight
                    );
                    context.lineTo(
                        touches[i].pageX - canvas.offsetLeft + HALF_PIXEL,
                        touches[i].pageY - canvas.offsetTop + HALF_PIXEL -
                            header.offsetHeight
                    );
                    context.closePath();
                    context.stroke();

                    drawPath[touches[i].identifier] = touches[i];
                }
            }
            ev.preventDefault();
        }

        /**
         * Registers canvas event listeners.
         *
         * @private
         */
        function addCanvasListeners() {
            canvas.addEventListener('touchstart', onCanvasTouchStart);
            canvas.addEventListener('touchend', onCanvasTouchEnd);
            canvas.addEventListener('touchmove', onCanvasTouchMove);
        }

        /**
         * Removes canvas event listeners.
         *
         * @private
         */
        function removeCanvasListeners() {
            canvas.removeEventListener('touchstart', onCanvasTouchStart);
            canvas.removeEventListener('touchend', onCanvasTouchEnd);
            canvas.removeEventListener('touchmove', onCanvasTouchMove);
        }

        /**
         * Handles views.init.visibility.change event
         * (document.visibilityState can be set to 'visible' or 'hidden').
         *
         * @private
         */
        function visibilityChange() {
            if (document.visibilityState !== 'visible') {
                removeCanvasListeners();
            } else {
                addCanvasListeners();
            }
        }

        /**
         * Registers view event listeners.
         *
         * @private
         */
        function bindEvents() {
            page.addEventListener('pageshow', onPageShow);
            optionsBtn.addEventListener('click', onOptionsBtnTap);
            clearBtn.addEventListener('click', onClearBtnTap);
            addCanvasListeners();
        }

        /**
         * Initializes canvas.
         *
         * @private
         */
        function initCanvas() {
            canvas = document.createElement('canvas');
            canvas.width = window.innerWidth;
            canvas.height = window.innerHeight - header.offsetHeight -
                footer.offsetHeight;
            canvas.setAttribute('id', 'main-canvas');
            content.appendChild(canvas);
            context = canvas.getContext('2d');
        }

        /**
         * Initializes module.
         *
         * @memberof views/main
         * @public
         */
        function init() {
            page = document.getElementById('main');
            content = document.getElementById('main-content');
            header = document.getElementById('main-header');
            footer = document.getElementById('main-footer');
            optionsBtn = document.getElementById('main-options-btn');
            clearBtn = document.getElementById('main-clear-btn');
            initCanvas();
            updateStrokeProperties();
            bindEvents();
        }

        e.listeners({
            'views.init.visibility.change': visibilityChange
        });

        return {
            init: init
        };
    }

});
